﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;

using System.Text;
using System.Windows.Forms;
using System.IO;
using WaterSimDCDC;
using WaterSimDCDC_Controls;
using System.Diagnostics;


namespace WaterSim_Tester
{

    public partial class WaterSimTestForm : Form
    {

        WaterSim_DB WSim;
        
        WaterSim_InputPanel WS_InputPanel;

        bool WSDB_Open = false;

        HelpAboutForm HADialog;

        public WaterSimTestForm()
        {
            InitializeComponent();

//            WSim = new WaterSim_DB("WaterSim_Output", "", null);
            WaterSim.CreateModelOutputFiles = false;
            //string MissingDataDir = "MissingDataTest\\WaterSim_4_0\\"; 
            //string remotedatadir =  "G:\\ASU\\DCDC\\Projects\\WaterSIM\\WaterSim_DCDC_50\\VS Projects\\REMOTEDATA\\WaterSim_4_0\\";
            WSim = new WaterSim_DB("WaterSim_Output", "App_Data\\WaterSim_4_0\\", null);
            //WSim = new WaterSim_DB("WaterSim_Output", MissingDataDir, null);
            //WSim = new WaterSim_DB("WaterSim_Output", remotedatadir, null);
            WSim.Simulation_Initialize();

 
            ShowMemory();
            WS_InputPanel = new WaterSim_InputPanel(WSim, InputPage.Width - 5, InputPage.Height - 5, 1, 1, InputPage);
            InputPage.Controls.Add(WS_InputPanel);
           
            HADialog = new HelpAboutForm(WSim.APiVersion, WSim.ModelBuild);
        }  // Form Constructor
        //-------------------------------------------------------------
        public string ReportTitle()
        {
            return ScenarioNameTextBox.Text + "   " + DateTime.Today.ToShortDateString() + "  " + DateTime.Now.ToShortTimeString() + "   WaterSim_DCDC Build " + WSim.ModelBuild;
        }
        //----------------------------------------------------------
        bool _InRunModel = false;
        internal void RunModel()
        {
            if (!_InRunModel)
            {
                _InRunModel = true;
                string ScName = ScenarioNameTextBox.Text;

                //string lineout = "";
                string ReportString = "";
                int ProviderSize = ProviderClass.NumberOfProviders;
                string[] AllStrings = new string[ProviderSize];
                
                // Resett the simulation
                WSim.Simulation_Initialize();
                
                // Fetch all the Input fields
                SimulationInputs SimInputs = WS_InputPanel.GetAllValues();
                SetAllModelValues(SimInputs);

                //SetStatusBar
                StatusLabel1.Text = "Running Simulation " + ScenarioNameTextBox.Text;
                StatusProgressBar.Maximum = WSim.Simulation_End_Year - WSim.Simulation_Start_Year;
                StatusProgressBar.Value = 1;
                StatusProgressBar.Visible = true;
              
                // Lock the model       
                WSim.LockSimulation();
                // Reset Memory Status
                ShowMemory();
                // Add a Title and Header
                OutputListBox.Items.Clear();
                OutputListBox.Items.Add(ReportTitle());
                //string[] header = new string[2];
                ReportHeader header = new ReportHeader();
                header = ReportClass.FullHeader(WSim, 20, false, true);
                OutputListBox.Items.Add(header[2]);
                OutputListBox.Items.Add(header[0]);
                OutputListBox.Items.Add(header[1]);


                if (OutputReportCheckBox.Checked)
                {
                    if (ContinuousCheckBox.Checked)
                        ReportString = ReportTextBox.Text;
                    else
                        ReportString = "";
                    ReportString += ReportTitle() + System.Environment.NewLine + ReportClass.FullHeader(WSim, 0, true, false)[0] + System.Environment.NewLine;
                }

                // OK run the model
                int runyear = 0;
                int cnt = 0;
                foreach (int year in WSim.simulationYears())
                {
                    runyear = WSim.Simulation_NextYear();
                    AllStrings = ReportClass.AnnualFullData(WSim,ScName, runyear, 20, false);
                    for (int i = 0; i < ProviderSize; i++)
                        OutputListBox.Items.Add(AllStrings[i]);
                    if (OutputReportCheckBox.Checked)
                    {
                        AllStrings = ReportClass.AnnualFullData(WSim, ScName, runyear, 20, true);
                        for (int i = 0; i < ProviderSize; i++)
                            ReportString += AllStrings[i];
                    }
                    StatusProgressBar.Value = cnt;
                    cnt++;
                }

                // Stop Model
                WSim.Simulation_Stop();

                StatusProgressBar.Visible = false;

                // output report
                if (OutputReportCheckBox.Checked)
                {
                    ReportTextBox.Text = ReportString + System.Environment.NewLine;
                }

                // Set output to default
                StatusLabel1.Text = "";

                tabControl1.SelectedTab = OutputTabPage;
                OutputTabPage.Select();

                // Allow entry
                _InRunModel = false;
            }
        }

        //-------------------------------------------------------------------------------------------------------
        bool _OneYearNotStarted = true;
        bool _inRunOneYear = false;
        int _OneYearStart = 0;
        int _OneYearStop = 0;
        //-------------------------------------------------------------------------------------------------------
        private void RunOneYear()
        {
            // Block Rentry
            if (!_inRunOneYear)
            {
                _inRunOneYear = true;

                int ProviderSize = ProviderClass.NumberOfProviders;
                ReportHeader lineout = new ReportHeader();
                string ReportString = "";
                string[] AllStrings = new string[ProviderSize];
                string ScName = ScenarioNameTextBox.Text;
                int _runYear = 0;
                //----------------------------
                if (_OneYearNotStarted)
                {
                    // Resett the simulation
                    WSim.Simulation_Initialize();

                    OutputListBox.Items.Clear();
                    OutputListBox.Items.Add(ReportTitle());
                    lineout = ReportClass.FullHeader(WSim, 20, false,true);
                    OutputListBox.Items.Add(lineout[0]);
                    OutputListBox.Items.Add(lineout[1]);

                    SetAllModelValues(WS_InputPanel.GetAllValues());
                    _OneYearStart = WSim.Simulation_Start_Year;
                    _OneYearStop = WSim.Simulation_End_Year;
                    if (OutputReportCheckBox.Checked)
                    {
                        if (ContinuousCheckBox.Checked)
                            ReportString = ReportTextBox.Text;
                        else
                            ReportString = "";
                        ReportString += ReportTitle() + System.Environment.NewLine + ReportClass.FullHeader(WSim, 0, true, false)[0] + System.Environment.NewLine;
                        ReportTextBox.Text = ReportString;
                    }
                    _OneYearNotStarted = false;
                } // _oneYearNotSTarted true
               
                // Set New Year
                    // Clear this just to be safe
                    ReportString = "";
                    // Run One year of model
                    SetAllModelValues(WS_InputPanel.GetAllValues());
                    // Lock the model       

                    WSim.LockSimulation();

                    _runYear = WSim.Simulation_NextYear();
 
                    AllStrings = ReportClass.AnnualFullData(WSim,ScName,  _runYear, 20, false);
                    for (int i = 0; i < ProviderSize; i++)
                        OutputListBox.Items.Add(AllStrings[i]);
                    if (OutputReportCheckBox.Checked)
                    {
                        ReportString = ReportTextBox.Text;
                        AllStrings = ReportClass.AnnualFullData(WSim, ScName, _runYear, 20, true);
                        for (int i = 0; i < ProviderSize; i++)
                            ReportString += AllStrings[i];
                        ReportTextBox.Text = ReportString;
                    }
                    if (_runYear == _OneYearStop)
                    {
                        WSim.Simulation_Stop();
                        MessageBox.Show("Simulation has ended, next run will restart simulation!");
                        _OneYearNotStarted = true;
                    }
                
                _inRunOneYear = false;
            }
        }
        private void RunModelButton_Click(object sender, EventArgs e)
        {
            RunModel();
        }
        //----------------------------------------------------------
        public void SetAllModelValues(SimulationInputs si)
        {
            WSim.ParamManager.SetSimulationInputs(si);
        }
        //-------------------------------------------------------------------------------------------------------
        private void Refresh_Inputs()
        {
            WS_InputPanel.Refresh_Inputs();
        }
        //-------------------------------------------------------------------------------------------------------

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        //-------------------------------------------------------------------------------------------------------
        private void TestDefaultValuesMenuItem_Click(object sender, EventArgs e)
        {
            int Value, Low, High = 0;
            bool test = false;
            string lineout, valstr, name = "";
            ModelParameterClass Mp;
            ProviderIntArray Values = new ProviderIntArray(0);
            string errString = "";
            
            //--------------------------
            WSim.Simulation_Initialize();
            //WSim.LockSimulation();
           // WSim.Simulation_NextYear();
            OutputListBox.Items.Clear();
            OutputListBox.Items.Add(ReportTitle());
            lineout = "Test " + "Model Input Parameter".PadRight(31, ' ') + "Low".ToString().PadLeft(8, ' ') + "High".ToString().PadLeft(8, ' ') + "Value".ToString().PadLeft(8, ' ');
            foreach (eProvider p in ProviderClass.providers())
            {
                name = ProviderClass.Label(p);
                if (name.Length > 9) name = name.Substring(0, 9);

                lineout += name.PadLeft(10, ' ');
            }
            OutputListBox.Items.Add(lineout);
            foreach (eModelParam emp in WSim.ParamManager.ModelParameters())
            {
                errString = "";
                Mp = WSim.ParamManager.Model_Parameter(emp);

                if (Mp.isInputParam)
                {
                    Low = Mp.LowRange;
                    High = Mp.HighRange;
                    name = Mp.Label;
                    if (name.Length > 30) name = name.Substring(0, 30);
                    lineout = name.PadRight(31, ' ') + Low.ToString().PadLeft(8, ' ') + High.ToString().PadLeft(8, ' ');
                    // check Base and Provider types
                    switch (Mp.ParamType)
                    {
                        case modelParamtype.mptInputBase:
                            Value = Mp.Value;
                            test = Mp.isBaseValueInRange(Value, ref errString);
                            lineout +=  Value.ToString().PadLeft(8, ' ');

                            break;
                        case modelParamtype.mptInputProvider:
                            Values = Mp.ProviderProperty.getvalues();
                            //valstr = "";
                            //foreach (eProvider p in ProviderClass.providers())
                            //{
                            //    test = Mp.isProviderValueInRange(Values[(int)p],p,ref errString[(int)emp]);
                            //    if (!test) break;
                            //}

                            valstr = "";
                            foreach (eProvider p in ProviderClass.providers())
                            {
                                string temp = "";
                                valstr += Values[(int)p].ToString().PadLeft(8, ' ');
                                test = Mp.isProviderValueInRange(Values[(int)p], p, ref temp);
                                if (!test)
                                {
                                    valstr += "* ";
                                    errString = temp;
                                }
                                else valstr += "  ";
                            }
                            lineout +=  "        " + valstr;
                            break;

                    }
                    if (!test)
                        lineout = "!!!> " + lineout + errString;
                    else
                        lineout = "     " + lineout;
                    OutputListBox.Items.Add(lineout);
                }
            }
            //----------------------------------Test
            lineout = "";
            //string temperr = "";
            //WSim.Simulation_Stop();
            //WSim.Simulation_Initialize();
            //OutputListBox.Items.Add("======================================");
            //OutputListBox.Items.Add("TESTING READ WRITE AFTER INITIALIZATION");

            //OutputListBox.Items.Add(" BASE Values");

            //foreach (ModelParameterClass MP in WSim.ParamManager.BaseInputs())
            //{
            //    if (!MP.TestReadWrite())
            //    {
            //        temperr = "Failed >> " + MP.Label;
            //    }
            //    else temperr = "          " + MP.Label;
            //    OutputListBox.Items.Add(temperr);
            //}
            //OutputListBox.Items.Add(" Provier Values");

            ////foreach (ModelParameterClass MP in WSim.ParamManager.ProviderInputs())
            ////{
            ////    {
            ////        WSim.ParamManager.Model_Parameter(eModelParam.epProvider_Demand_Option).Value = 4;
            ////    }
            ////    if (!MP.TestReadWrite())
            ////    {
            ////        temperr = "Failed >> " + MP.Label;
            ////    }
            ////    else temperr = "          " + MP.Label;
            ////    OutputListBox.Items.Add(temperr);
            ////}
            
            //WSim.Simulation_Initialize();
            //WSim.Simulation_NextYear();
            //OutputListBox.Items.Add("======================================");
            //OutputListBox.Items.Add("TESTING READ WRITE AFTER One YEar Run");

            //OutputListBox.Items.Add(" BASE Values");

            //foreach (ModelParameterClass MP in WSim.ParamManager.BaseInputs())
            //{
            //    if (!MP.TestReadWrite())
            //    {
            //        temperr = "Failed >> " + MP.Label;
            //    }
            //    else temperr = "          " + MP.Label;
            //    OutputListBox.Items.Add(temperr);
            //}
            //OutputListBox.Items.Add(" Provier Values");

            //foreach (ModelParameterClass MP in WSim.ParamManager.ProviderInputs())
            //{
            //    {
            //        WSim.ParamManager.Model_Parameter(eModelParam.epProvider_Demand_Option).SetInt(4);
            //    }
            //    if (!MP.TestReadWrite())
            //    {
            //        temperr = "Failed >> " + MP.Label;
            //    }
            //    else temperr = "          " + MP.Label;
            //    OutputListBox.Items.Add(temperr);
            //}

            //WSim.Simulation_Initialize();
            //WSim.Simulation_NextYear();
            //WSim.Simulation_Stop();
            //OutputListBox.Items.Add("======================================");
            //OutputListBox.Items.Add("TESTING READ WRITE AFTER Stop Simulation");
            //OutputListBox.Items.Add("BASE Values");

            //foreach (ModelParameterClass MP in WSim.ParamManager.BaseInputs())
            //{

            //    if (!MP.TestReadWrite())
            //    {
            //        temperr = "Failed >> " + MP.Label;
            //    }
            //    else temperr = "          " + MP.Label;
            //    OutputListBox.Items.Add(temperr);
            //}
            //OutputListBox.Items.Add("Provier Values");

            //foreach (ModelParameterClass MP in WSim.ParamManager.ProviderInputs())
            //{
            //    if (MP.ModelParam == eModelParam.epUse_GPCD)
            //    {
            //        WSim.ParamManager.Model_Parameter(eModelParam.epProvider_Demand_Option).SetInt(4);
            //    }
            //    if (!MP.TestReadWrite())
            //    {
            //        temperr = "Failed >> " + MP.Label;
            //    }
            //    else temperr = "          " + MP.Label;
            //    OutputListBox.Items.Add(temperr);
            //}
    
        }
        internal long Last_Memory = 0;
        //-----------------------------------------------
        internal void ShowMemory()
        {
            Process MyProcess = System.Diagnostics.Process.GetCurrentProcess();

            long change = 0;
            long Process_Memory = MyProcess.PrivateMemorySize64;

            long GC_Memory = System.GC.GetTotalMemory(true);
            change = GC_Memory - Last_Memory;
            MemoryLabel.Text = "Memory - System: " + String.Format("{0:0,0}", Process_Memory) + " Managed: " + String.Format("{0:0,0}", GC_Memory) + "  Delta: " + String.Format("{0:0,0}", change) + "  ";
            Last_Memory = GC_Memory;
            Application.DoEvents();
        }


        //------------------------------------------------------------------
        private void CopyOutputButton_Click(object sender, EventArgs e)
        {
            string temp = "";
            int cnt = 0;
            cnt = OutputListBox.Items.Count;
            for (int i = 0; i < cnt; i++)
                temp = temp + OutputListBox.Items[i] + System.Environment.NewLine;
            Clipboard.SetData(DataFormats.Text, temp);
        }
        //-------------------------------------------------------------------
        private void forceNewModelToolStripMenuItem_Click(object sender, EventArgs e)
        {
        }
        //----------------------------------------------------------
        int RepeatSize = 50;
        //------------------------------------------------------------------------
        private void rRRepeatRunMenuItem_Click(object sender, EventArgs e)
        {
            // Run Constants
            int maxruns = RepeatSize;
            int Numberoftestyears = 3;
            //Test Data
            long memteststart = System.GC.GetTotalMemory(true);
            DateTime startTime, endTime;
            int[] testyear = new int[Numberoftestyears];
            int[, ,] testvalues = new int[Numberoftestyears, maxruns, WSim.ParamManager.NumberOfParameters(modelParamtype.mptOutputBase)];
            int[,] testresults = new int[Numberoftestyears, WSim.ParamManager.NumberOfParameters(modelParamtype.mptOutputBase)];
            // Local
            string lineout = "";
            string temp = "";
            string resultstr = "";
            ModelParameterClass Mp;
            int progresstest = 0;
            if (maxruns < 100)
                progresstest = 2;
            else
                progresstest = maxruns / 50;
            // Fetch all the Input fields
            SetAllModelValues(WS_InputPanel.GetAllValues());
            // calculate test years
            int start = WSim.Simulation_Start_Year;
            int end = WSim.Simulation_End_Year;
            int span = end - start;
            if (span < Numberoftestyears)
            {
                MessageBox.Show("Run Span must be more than " + Numberoftestyears.ToString() + "years to do Repeat test");
                WSim.Simulation_Stop();
                return;
            }
            int inc = span / Numberoftestyears;
            for (int i = 0; i < Numberoftestyears; i++)
                testyear[i] = start + (i * inc);

            // STart Status Report
            StatusLabel1.Text = "Running Repeat Model Test";
            StatusProgressBar.Maximum = maxruns;
            StatusProgressBar.Minimum = 0;
            StatusProgressBar.Value = 0;
            StatusProgressBar.Visible = true;

            // Run the Model and collect results
            int runyear = 0;
            int pindex = 0;
            startTime = DateTime.Now;
            for (int runi = 0; runi < maxruns; runi++)
            {
                // Reset the simulation
                WSim.Simulation_Initialize();
                // Fetch all the Input fields
                SetAllModelValues(WS_InputPanel.GetAllValues());


                runyear = 0;

                foreach (int year in WSim.simulationYears())
                {
                    runyear = WSim.Simulation_NextYear();
                    for (int i = 0; i < Numberoftestyears; i++)
                    {
                        if (testyear[i] == runyear)
                        {
                            pindex = 0;
                            foreach (eModelParam emp in WSim.ParamManager.ModelParameters())
                            {
                                Mp = WSim.ParamManager.Model_Parameter(emp);
                                if (Mp.ParamType == modelParamtype.mptOutputBase)
                                {
                                    testvalues[i, runi, pindex] = Mp.Value;
                                    pindex++;
                                }
                            }
                        } // if test year
                    }  // i number of year
                } // year simulationyears 
                WSim.Simulation_Stop();
                if ((runi % progresstest) == 0)
                {
                    StatusProgressBar.Value = runi;
                    Application.DoEvents();
                }
            } // runi  max runs
            endTime = DateTime.Now;
            TimeSpan Span = endTime - startTime;
            double PerRunmsec = (Span.TotalMilliseconds / maxruns) / 1000;
            long memtestend = System.GC.GetTotalMemory(true);
            long memdelta = memtestend - memteststart;

            StatusProgressBar.Visible = false;
            StatusLabel1.Text = "Analyzing Test Results";
            Application.DoEvents();

            // Analyze Results

            for (int i = 0; i < Numberoftestyears; i++)
            {
                pindex = 0;
                foreach (eModelParam emp in WSim.ParamManager.ModelParameters())
                {
                    Mp = WSim.ParamManager.Model_Parameter(emp);
                    if (Mp.ParamType == modelParamtype.mptOutputBase)
                    {
                        for (int runi = 1; runi < maxruns; runi++)
                        {
                            if (testvalues[i, runi, pindex] != testvalues[i, 0, pindex]) testresults[i, pindex]++;
                        }
                        pindex++;
                    } // if maprtype
                } // emp Modelparamerts
            } // i numberof testyears
            //  Got results, now report
            OutputListBox.Items.Clear();
            OutputListBox.Items.Add("Model Repeat Run Test");
            OutputListBox.Items.Add(" Runs = " + maxruns.ToString() + "  Years Tested = " + Numberoftestyears.ToString());
            OutputListBox.Items.Add("Memory - Start: " + String.Format("{0:0,0}", memteststart) + "  End: " + String.Format("{0:0,0}", memtestend) + "  Delta: " + String.Format("{0:0,0}", memdelta));
            OutputListBox.Items.Add("Start : " + startTime.ToShortTimeString() + "   End: " + endTime.ToShortTimeString() + "  Duration: " + Span.ToString() + " per Run: " +
                String.Format("{0:0.000}", PerRunmsec) + " secs");
            OutputListBox.Items.Add("Failed test = !!!>");
            lineout = "Test " + "Model ParameterRuns".PadRight(22, ' ');
            for (int i = 0; i < Numberoftestyears; i++)
            {
                lineout += testyear[i].ToString().PadLeft(10, ' ');
            }
            OutputListBox.Items.Add(lineout);

            pindex = 0;
            bool test = true;
            foreach (eModelParam emp in WSim.ParamManager.ModelParameters())
            {
                test = true;
                Mp = WSim.ParamManager.Model_Parameter(emp);
                if (Mp.ParamType == modelParamtype.mptOutputBase)
                {
                    resultstr = "";
                    for (int i = 0; i < Numberoftestyears; i++)
                    {
                        if (testresults[i, pindex] > 0)
                        {
                            test = false;
                        }
                        resultstr += String.Format("{0:0,0}", testresults[i, pindex]).PadLeft(10, ' ');
                    }
                    if (!test) lineout = "!!!> ";
                    else lineout = "     ";
                    temp = Mp.Label;
                    if (temp.Length > 20) temp = temp.Substring(0, 20);
                    lineout += temp.PadRight(22, ' ') + resultstr;
                    OutputListBox.Items.Add(lineout);
                }
                StatusLabel1.Text = "";
            }

            // now all results
            lineout = "  Year   Run";
            foreach (eModelParam emp in WSim.ParamManager.ModelParameters())
            {
                Mp = WSim.ParamManager.Model_Parameter(emp);
                if (Mp.ParamType == modelParamtype.mptOutputBase)
                {
                    temp = Mp.Fieldname;
                    if (temp.Length > 10) temp = temp.Substring(0, 10);
                    lineout = lineout + temp.PadLeft(12, ' ');
                }
            }
            OutputListBox.Items.Add(lineout);
            for (int year = 0; year < Numberoftestyears; year++)
            {
                for (int runi = 0; runi < maxruns; runi++)
                {
                    lineout = testyear[year].ToString().PadRight(6, ' ') + runi.ToString().PadLeft(5, ' ');
                    pindex = 0;
                    foreach (eModelParam emp in WSim.ParamManager.ModelParameters())
                    {
                        Mp = WSim.ParamManager.Model_Parameter(emp);
                        if (Mp.ParamType == modelParamtype.mptOutputBase)
                        {
                            lineout += String.Format("{0:0,0}", testvalues[year, runi, pindex]).PadLeft(12, ' ');
                        }
                        pindex++;
                    }
                    OutputListBox.Items.Add(lineout);
                }
            }
            ShowMemory();
        }

        private void textBox1_Leave(object sender, EventArgs e)
        {
            string temp = "";
            temp = RepeatTestTextBox.Text;
            int value = Convert.ToInt16(temp);
            RepeatSize = value;
        }


        private void textBox1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Return)
            {
                string temp = "";
                temp = RepeatTestTextBox.Text;
                int value = Convert.ToInt16(temp);
                RepeatSize = value;
            }

        }
        //-------------------------------------------------------------------------------------
        bool _inMemoryTimeTest = false;
        private void memoryTimeTestMenuItem_Click(object sender, EventArgs e)
        {
            if (!_inMemoryTimeTest)
            {
                // prevent re_entry
                _inMemoryTimeTest = true;

                // Setup local
                int maxruns = RepeatSize;
                long mstart, mend, mdelta, mper = 0;
                DateTime tstart, tend;
                TimeSpan tdelta;
                double tper = 0;

                int BaseInputSize = WSim.ParamManager.NumberOfParameters(modelParamtype.mptInputBase);
                int BaseOutputSize = WSim.ParamManager.NumberOfParameters(modelParamtype.mptOutputBase);
                int ProviderInputSize = WSim.ParamManager.NumberOfParameters(modelParamtype.mptInputProvider);
                int ProviderOutputSize = WSim.ParamManager.NumberOfParameters(modelParamtype.mptOutputProvider);


                AnnualSimulationResults AnnualResults = new AnnualSimulationResults(BaseOutputSize, ProviderOutputSize, BaseInputSize, ProviderInputSize);


                //================= BASE TEST ==================
                // Report Status and clear everthing out to extent possible.
                StatusLabel1.Text = "Running Memory Test (estimate " + maxruns.ToString() + " seconds) " + DateTime.Now.ToShortTimeString();
                OutputListBox.Items.Clear();

                Application.DoEvents();

                // Grab Data and Start
                mstart = System.GC.GetTotalMemory(true);
                tstart = DateTime.Now;

                // Do runs as fast as possible
                for (int i = 0; i < maxruns; i++)
                {
                    WSim.Simulation_Initialize();
                    WSim.LockSimulation();
                    WSim.Simulation_AllYears();
                    WSim.Simulation_Stop();
                }

                // Collect memory and time
                tend = DateTime.Now;
                mend = System.GC.GetTotalMemory(true);

                // Figure it out and report
                tdelta = tend - tstart;
                tper = tdelta.TotalSeconds / maxruns;
                mdelta = mend - mstart;
                mper = mdelta / maxruns;

                OutputListBox.Items.Add("Memory and Time Test " + DateTime.Now.ToShortDateString());
                OutputListBox.Items.Add("      Runs : " + maxruns.ToString());
                OutputListBox.Items.Add("      Time - Start: " + tstart.ToShortTimeString() + "  End: " + tend.ToShortTimeString() + "  Duration : " + tdelta.ToString() + "  Per Run: " +
                    String.Format("{0:0.000}", tper));
                OutputListBox.Items.Add("      Memory - Start: " + String.Format("{0:0,0}", mstart) + "  End: " + String.Format("{0:0,0}", mend) + "  Delta: " + String.Format("{0:0,0}", mdelta) +
                    "  Per Run: " + String.Format("{0:0,0}", mper));

                //================= With Memory TEST ==================
                // Report Status and clear everthing out to extent possible.
                StatusLabel1.Text = "Running Memory Test With Fetch (estimate " + maxruns.ToString() + " seconds) " + DateTime.Now.ToShortTimeString();
                Application.DoEvents();
                int yearMax = (WSim.Simulation_End_Year - WSim.Simulation_Start_Year) + 1;

                // Grab Data and Start
                mstart = System.GC.GetTotalMemory(true);
                tstart = DateTime.Now;

                // Do runs by year
                for (int i = 0; i < maxruns; i++)
                {
                    WSim.Simulation_Initialize();
                    WSim.LockSimulation();
                    for (int j = 0; j < yearMax; j++)
                    {
                        WSim.Simulation_NextYear();
                        WSim.FastGetAnnualData(ref AnnualResults);
                    }
                    WSim.Simulation_Stop();
                }

                // Collect memory and time
                tend = DateTime.Now;
                mend = System.GC.GetTotalMemory(true);

                // Figure it out and report
                tdelta = tend - tstart;
                tper = tdelta.TotalSeconds / maxruns;
                mdelta = mend - mstart;
                mper = mdelta / maxruns;

                OutputListBox.Items.Add("Memory and Time Test With Fetch " + DateTime.Now.ToShortDateString());
                OutputListBox.Items.Add("      Runs : " + maxruns.ToString());
                OutputListBox.Items.Add("      Time - Start: " + tstart.ToShortTimeString() + "  End: " + tend.ToShortTimeString() + "  Duration : " + tdelta.ToString() + "  Per Run: " +
                    String.Format("{0:0.000}", tper));
                OutputListBox.Items.Add("      Memory - Start: " + String.Format("{0:0,0}", mstart) + "  End: " + String.Format("{0:0,0}", mend) + "  Delta: " + String.Format("{0:0,0}", mdelta) +
                    "  Per Run: " + String.Format("{0:0,0}", mper));


                //================= With Report Building TEST ==================
                // Report Status and clear everthing out to extent possible.
                StatusLabel1.Text = "Running Memory Test With Report Building (estimate " + maxruns.ToString() + " seconds) " + DateTime.Now.ToShortTimeString();
                Application.DoEvents();
                yearMax = (WSim.Simulation_End_Year - WSim.Simulation_Start_Year) + 1;
                int runyear = WSim.Simulation_Start_Year;
                // Grab Data and Start
                mstart = System.GC.GetTotalMemory(true);
                tstart = DateTime.Now;
                ProviderStringArray AllStrings = new ProviderStringArray("");
                // Do runs by year
                for (int i = 0; i < maxruns; i++)
                {
                    WSim.Simulation_Initialize();
                    WSim.LockSimulation();
                    for (int j = 0; j < yearMax; j++)
                    {
                        WSim.Simulation_NextYear();
                        AllStrings.Values = ReportClass.AnnualFullData(WSim,"MEMORYTEST", runyear, 20, true);
                        runyear++;
                    }
                    WSim.Simulation_Stop();
                }

                // Collect memory and time
                tend = DateTime.Now;
                mend = System.GC.GetTotalMemory(true);

                // Figure it out and report
                tdelta = tend - tstart;
                tper = tdelta.TotalSeconds / maxruns;
                mdelta = mend - mstart;
                mper = mdelta / maxruns;

                OutputListBox.Items.Add("Memory and Time Test With Fetch " + DateTime.Now.ToShortDateString());
                OutputListBox.Items.Add("      Runs : " + maxruns.ToString());
                OutputListBox.Items.Add("      Time - Start: " + tstart.ToShortTimeString() + "  End: " + tend.ToShortTimeString() + "  Duration : " + tdelta.ToString() + "  Per Run: " +
                    String.Format("{0:0.000}", tper));
                OutputListBox.Items.Add("      Memory - Start: " + String.Format("{0:0,0}", mstart) + "  End: " + String.Format("{0:0,0}", mend) + "  Delta: " + String.Format("{0:0,0}", mdelta) +
                    "  Per Run: " + String.Format("{0:0,0}", mper));


              
                   //================= With DB  TEST ==================
                if(WSDB_Open)
                {
                    string tablename = "MEMORYTEST" + DateTime.Now.Ticks.ToString();
                    StatusLabel1.Text = "Running Memory Test With DB Simulation (estimate " + maxruns.ToString() + " seconds) " + DateTime.Now.ToShortTimeString();
                    Application.DoEvents();

                    
                    DataTable DT = new DataTable();
                   // get the table name
                   WSim.CreateNewDataTable(tablename);

                    // Ok fill the data table with all teh relvant fields
                    OleDbDataAdapter NewRawTableAdapter = new OleDbDataAdapter("Select * from " + '[' + tablename + ']', WSim.DbConnection);
                    NewRawTableAdapter.Fill(DT);

                    // Grab Data and Start
                    mstart = System.GC.GetTotalMemory(true);
                    tstart = DateTime.Now;

                    for (int i = 0; i < maxruns; i++)
                    {
                        // initialize a db simulation, turn control over to WaterSim_DB
                        WSim.Simulation_Initialize(DT, ScenarioNameTextBox.Text);
                        // Fetch all the Input fields
                        SetAllModelValues(WS_InputPanel.GetAllValues());
                        // run each year of simulation db
                        foreach (int year in WSim.simulationYears())
                            WSim.Simulation_NextYear();
                        // stop the simulation (which adds all data to data table
                        WSim.Simulation_Stop();
                        // OK I now have control, update the database
                        OleDbCommandBuilder TableUpdateBuilder = new OleDbCommandBuilder(NewRawTableAdapter);
                        NewRawTableAdapter.Update(DT);
                    }

                    // Collect memory and time
                    tend = DateTime.Now;
                    mend = System.GC.GetTotalMemory(true);

                    // Figure it out and report
                    tdelta = tend - tstart;
                    tper = tdelta.TotalSeconds / maxruns;
                    mdelta = mend - mstart;
                    mper = mdelta / maxruns;

                    OutputListBox.Items.Add("Memory and Time With Simulation_DB  " + DateTime.Now.ToShortDateString());
                    OutputListBox.Items.Add("      Runs : " + maxruns.ToString());
                    OutputListBox.Items.Add("      Time - Start: " + tstart.ToShortTimeString() + "  End: " + tend.ToShortTimeString() + "  Duration : " + tdelta.ToString() + "  Per Run: " +
                        String.Format("{0:0.000}", tper));
                    OutputListBox.Items.Add("      Memory - Start: " + String.Format("{0:0,0}", mstart) + "  End: " + String.Format("{0:0,0}", mend) + "  Delta: " + String.Format("{0:0,0}", mdelta) +
                        "  Per Run: " + String.Format("{0:0,0}", mper));



                }

                StatusLabel1.Text = "";
                _inMemoryTimeTest = false;
            }

        }
        //----------------------------------------------------------------------------------------
        internal void TestRangeChecks()
        {
            OutputListBox.Items.Clear();

            //bool test = false;
            int testvalue = 0;
            foreach (ModelParameterClass MP in WSim.ParamManager.BaseInputs())
            {
                testvalue = MP.HighRange + 10;
               


            }
            foreach (ModelParameterClass MP in WSim.ParamManager.ProviderInputs())
            {

            }
        }
        //----------------------------------------------------------------------------------------
        private void RunInputsMenuItem_Click(object sender, EventArgs e)
        {
            RunModel();
        }

        //----------------------------------------------------------------------------------------
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {

        }

        //----------------------------------------------------------------------------------------
        private void ProviderComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            //eProvider pSelected = ProviderClass.provider(ProviderComboBox.SelectedIndex);
            //    foreach (WaterSim_Provider_InputTextBox piTextBox in ProviderInputList)
            //     piTextBox.ModelParameterProvider = pSelected;

        }
        //----------------------------------------------------------------------------------------
        private void CopyReportButton_Click(object sender, EventArgs e)
        {
            Clipboard.SetData(DataFormats.Text, ReportTextBox.Text);
        }

        //----------------------------------------------------------------------------------------
        private bool OpenDB()
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    WSim.DatabaseFilename = openFileDialog1.FileName;
                    WSDB_Open = true;
                    RunDBSimulationButton.Visible = true;
                    loadScenarioToolStripMenuItem.Enabled = true;
                }
                catch
                {
                    WSDB_Open = false;
                    RunDBSimulationButton.Visible = false;
  
                    MessageBox.Show(" Unable  to open " + openFileDialog1.FileName);
                }
            }
            return WSDB_Open;
        }
        //--------------------------------------------------------------------------------------
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenDB();
        }
        //--------------------------------------------------------------------------------------
        void LoadScenario()
        {
            if (WSDB_Open)
            {
                TablenameDialog tableDialog = new TablenameDialog();
                string tablename = "";
                DataTable DT = new DataTable();
                // get the table name
                if (tableDialog.ShowDialog(WSim.DbConnection, false, true) == System.Windows.Forms.DialogResult.OK)
                {
                    tablename = tableDialog.Tablename;
                    if (tablename != "")
                    {
                        try
                        {
                            OleDbDataAdapter TableAdapter = new OleDbDataAdapter("Select * from " + '[' + tablename + ']', WSim.DbConnection);
                            TableAdapter.Fill(DT);
                            TableAdapter.Dispose();

                            SimulationInputs SI = WSim.ParamManager.GetSimulationInputs();
                            if (WSim.LoadScenario_DB("", 0, DT, ref SI))
                            {
                                if (WSim.ParamManager.SetSimulationInputs(SI))
                                {
                                    WS_InputPanel.Refresh_Inputs();
                                }

                            }
                        }
                        catch (Exception e)
                        {
                            MessageBox.Show("Whoops " + e.Message);
                        }
                    }
                }
            }
            else
            {

            }

        }
        //--------------------------------------------------------------------------------------
        // private void button3_Click(object sender, EventArgs e)
        //{
        //    ModelParameterProviderArray MPA = new ModelParameterProviderArray(10);
        //}

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (HADialog == null)
                HADialog = new HelpAboutForm(WSim.APiVersion, WSim.ModelBuild);
            HADialog.Mode = eHelpAboutMode.haAbout;
            HADialog.ShowDialog();
        }
        //--------------------------------------------------------------------------------------
 
        private void helpToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (HADialog == null)
                HADialog = new HelpAboutForm(WSim.APiVersion, WSim.ModelBuild);
            HADialog.Mode = eHelpAboutMode.haHelp;
            HADialog.ShowDialog();
        }
        //--------------------------------------------------------------------------------------
 
        public void RunAllyearsDB()
        {
            if (WSDB_Open)
            {
                TablenameDialog tableDialog = new TablenameDialog();
                string runtablename = "";
                DataTable DT = new DataTable();
                // get the table name
                if (tableDialog.ShowDialog(WSim.DbConnection, false, false) == System.Windows.Forms.DialogResult.OK)
                {
                    runtablename = tableDialog.Tablename;
                    if (tableDialog.isNewTablename)
                        WSim.CreateNewDataTable(runtablename);

                }
                if (runtablename != "")
                {

                    StatusLabel1.Text = "Running DB Simulation";

                    // Ok fill the data table with all teh relvant fields
                    OleDbDataAdapter NewRawTableAdapter = new OleDbDataAdapter("Select * from " + '[' + runtablename + ']', WSim.DbConnection);
                    NewRawTableAdapter.Fill(DT);

                    // initialize a db simulation, turn control over to WaterSim_DB
                    WSim.Simulation_Initialize(DT, ScenarioNameTextBox.Text);
                    // Fetch all the Input fields
                    SetAllModelValues(WS_InputPanel.GetAllValues());

                    // run each year of simulation db
                    foreach (int year in WSim.simulationYears())
                        WSim.Simulation_NextYear();
                    // stop the simulation (which adds all data to data table
                    WSim.Simulation_Stop();
                    // OK I now have control, update the database
                    OleDbCommandBuilder TableUpdateBuilder = new OleDbCommandBuilder(NewRawTableAdapter);
                    NewRawTableAdapter.Update(DT);
                    // all done
                    StatusLabel1.Text = "Done";
                    dbTableGridView.DataSource = DT;

                }
            }
            else
            {
                MessageBox.Show("A Database must first be opened");
                OpenDB();
            }

        }
        //--------------------------------------------------------------
        bool _InTestProcess = false;
        void TestPrePostProcess()
        {
            if (!_InTestProcess)
            {
                _InTestProcess = true;
                // Create the tracker
                TrackProviderDeficits TPD = new TrackProviderDeficits();
               

                // Resett the simulation
                WSim.Simulation_Initialize();

                // Add tracker to post process
                WSim.ProcessManager.AddProcess(TPD);

                // Fetch all the Input fields
                SetAllModelValues(WS_InputPanel.GetAllValues());


                //SetStatusBar
                StatusLabel1.Text = "Running Simulation " + ScenarioNameTextBox.Text;
              
                // Lock the model       
                WSim.LockSimulation();
                // Reset Memory Status
                ShowMemory();
                // Clear box
                OutputListBox.Items.Clear();
                // OK run the model
                WSim.Simulation_AllYears();
                // Stop Model
                WSim.Simulation_Stop();

                // OK write tracked results
                if (WSim.ProcessManager.isProcessError)
                {

                }
                int cnt = 0;
                double tot = 0.0;
                foreach (eProvider ep in ProviderClass.providers())
                {
                    cnt = TPD.CountList[ep];
                    tot = TPD.TotalList[ep];
                    OutputListBox.Items.Add(ProviderClass.Label(ep).PadRight(30,'.')+cnt.ToString().PadLeft(10,' ')+tot.ToString().PadLeft(10,' '));


                }
                // Aldone with this Process, so dispose
                TPD.Dispose();

                // Set output to default
                StatusLabel1.Text = "";

                tabControl1.SelectedTab = OutputTabPage;
                OutputTabPage.Select();


                // Allow entry
                _InTestProcess = false;
            }  // tracker disposed of
        }

        
       //--------------------------------------------------------------
        void SaveReport()
        {
            if (ReportTextBox.Text == "")
            {
                MessageBox.Show("There is no report to save.");
            }
            else
            {
                if (saveFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    ReportTextBox.SaveFile(saveFileDialog1.FileName);

                }
            }
        }
        //---------------------------------------------------------------
        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {

        }

        private void allYearsDBToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RunAllyearsDB();
        }

        private void createReportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OutputReportCheckBox.Checked = !OutputReportCheckBox.Checked;
            createReportToolStripMenuItem.Checked = OutputReportCheckBox.Checked;
        }

        private void OutputReportCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            createReportToolStripMenuItem.Checked = OutputReportCheckBox.Checked;
        }

        private void continuousReportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ContinuousCheckBox.Checked = !ContinuousCheckBox.Checked;
            continuousReportToolStripMenuItem.Checked = ContinuousCheckBox.Checked;

        }

        private void ContinuousCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            continuousReportToolStripMenuItem.Checked = ContinuousCheckBox.Checked;

        }

        private void RunDBSimulationButton_Click(object sender, EventArgs e)
        {
            RunAllyearsDB();
        }

        private void RunOneYearButton_Click(object sender, EventArgs e)
        {
            RunOneYear();
        }

        private void modelDebugWriteToolStripMenuItem_Click(object sender, EventArgs e)
        {
           
        }

        private void SaveReportMenuItem_Click(object sender, EventArgs e)
        {
            SaveReport();
        }

        private void OpenDBButton_Click(object sender, EventArgs e)
        {
            OpenDB();
        }

        private void allYearsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RunModel();
        }

        private void nextYearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RunOneYear();
        }

        private void SaveReportButton_Click(object sender, EventArgs e)
        {
            SaveReport();
        }

        private void testSetRangeCheckToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TestRangeChecks();

        }

        private void postAndPreProcessToolStripMenuItem_Click(object sender, EventArgs e)
        {
            TestPrePostProcess();
        }

        private void loadScenarioToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LoadScenario();
        }
        //----------------------------------------------------------------------------------------


        //==================================================

    }

}