﻿using System;
using System.Collections.Generic;
using System.Drawing;

using System.Text;
using System.Windows.Forms;
using WaterSimDCDC;
/**********************************************************************************
    WaterSimDCDC Regional Water Demand and Supply Model
	Version 5.0

	WaterSim_Controls Version 5.6 7/25/11  Quay

    Copyright (C) 2011 , The Arizona Board of Regents
	on behalf of Arizona State University
	
    All rights reserved.

	Developed by the Decision Center for a Desert City
	Lead Model Development - David Sampson <dasamps1@asu.edu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 3 as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
    Three Control classes designed to be used with the WaterSim and WaterSimDB classes.
  
    WaterSim_Base_InputTextBox,  WaterSim_Provider_InputTextBox, WaterSim_InputPanel


 *******************************************************************************/
namespace WaterSimDCDC_Controls
{
    //=================================================
    /// <summary>   Delegate for handling OnModelParameterSet events. </summary>
    /// <param name="source">   Source for the event, this textbox. </param>
    /// <param name="e">        Event information to send to registered event handlers. </param>
    /// <remarks> OnModelParameterSet events are issued when the textbox sets a model parameter.  Only occurs if 
    /// 		          values are NOT stored locally.  </remarks>
    public delegate void OnModelParameterSetEventHandler (object source, System.EventArgs e);
   
    //=================================================
    abstract class  WaterSim_InputTextBox : TextBox
    {
        protected  bool FStoreLocal = true;
        protected  int FValue = 0;  // This is the TextBox maintained internal value, must call SetValue() to actually set Model Parameter Value
        protected  ModelParameterClass Fmp;
        protected  bool _isMPSet = false;
        //----------------------------------------------------------------------

        /// <summary>   Default constructor. </summary>
        /// <remarks> Values stored locally, no error checking, no events</remarks>
        public WaterSim_InputTextBox() : base ()  { }
        //----------------------------------------------------------------------

        /// <summary>   Constructor. </summary>
        /// <param name="aModelParameter">  a ModelParameter. </param>
        /// <param name="SoreLocal">        true to store value local, false to store in Model Parameter. </param>
        /// <remarks> Storing local does not access model during user input, only on initialization or on a reload cal or event.  Code must fetch the data at set the model parameters.  
        /// 		           Storing in model access the model and sets the model parameter each time a value is changed.</remarks>
        public WaterSim_InputTextBox(ModelParameterClass aModelParameter, bool StoreLocal)
            : base()
        {
            FStoreLocal = StoreLocal;
            _setmp(aModelParameter);
            this.KeyDown += new KeyEventHandler(InputTextBox_KeyDown);
            this.Leave += new EventHandler(InputTextBox_Leave);
            GetModelValue();
        }
       //---------------------------------------------------------
        abstract protected bool _setmp(ModelParameterClass mp);
       //---------------------------------------------------------

        /// <summary>   Gets or sets the ModelParameter. </summary>
        ///
        /// <value> The model parameter. </value>
        public ModelParameterClass ModelParameter
        {
            get { return Fmp;  }
            set { _setmp(value); }
        }
    //---------------------------------------------------------
        protected virtual void InputTextBox_KeyDown(object sender, KeyEventArgs e)
        {
       
            if (e.KeyCode == Keys.Return)
            { 
                int value = 0;
                string temp = "";
                temp = this.Text;
                try
                {
                    value = Convert.ToInt16(temp);
                }
                catch
                {
                    value = 0;
                }
                    if (value!=FValue)
                    SaveValue(value);
            }
        }
     //------------------------------
        protected virtual void InputTextBox_Leave(object sender, EventArgs e)
        {
            int value = 0;
            string temp = "";
            temp = this.Text;
            try
            {
                value = Convert.ToInt16(temp);
            }
            catch 
            {
                value = 0;
            }
            if (value != FValue)
                SaveValue(value);
        }
        //---------------------------------------
        abstract protected void SetValue(int value);
        //---------------------------------------
        abstract protected bool ValueInRange(int value);
        //------------------------
        abstract protected void SetModelValue();
        // Sets the actual model parameter value(s)
        //------------------------
        abstract protected void GetModelValue();
        // Gets the actual model value(s)
        //------------------------------------------------
          protected void SaveValue(int value)
          {
              if (Fmp!=null)
              {
                  if (Fmp.ShouldCheckRange())
                  {
                      if (ValueInRange(value))
                      {
                          SetValue(value);
                      }
                      else
                      {
                      //    MessageBox.Show("Invalid Parameter: Value = " + value.ToString() + " - " + Fmp.Label + " must be within range " + Fmp.LowRange.ToString() + " to " + Fmp.HighRange.ToString());
                            this.Text = FValue.ToString();
                          this.Focus();
                      }
                  }
                  else
                  {
                      SetValue(value);
                  }
              }
          }
        //-----------------------------------------------

          /// <summary> Reload event handler for ReloadEvents, reloads the parameter from the model. </summary>
          ///
          /// <remarks> Ray, 8/5/2011. </remarks>
          ///
          /// <param name="source"> Source ModelParamter for the event. </param>
          /// <param name="e">      Event information. </param>
          public virtual void ReloadEventHandler(object source, System.EventArgs e)
          {
              if (e is ModelParameterEventArgs)
              {
                  ReloadModelValue();
              }
          }
        //-----------------------------------------------------------------

        /// <summary>   Reloads the parameter value from the current model value. </summary>
        ///
        /// <remarks>   Ray, 8/5/2011. </remarks>

        public virtual void ReloadModelValue()
        {
            GetModelValue();
        }
        //---------------------------------------------------------------------

        /// <summary>   Gets or sets the Parameter value saved by the text box. </summary>
        ///
        /// <value> The current int value . </value>
        //-------------------------------------------------------
        public int Value
        {
            get { return FValue; }
            set { SetValue(value); }

        }
    
    }
    //=================================================
    class WaterSim_Base_InputTextBox : WaterSim_InputTextBox    
    {

        //------------------------
        /// <summary>   Default constructor. </summary>
        /// <remarks> Values stored locally, no error checking, no events</remarks>
 
        public WaterSim_Base_InputTextBox() : base() { }
        //------------------------

        /// <summary>   Constructor. </summary>
        /// <param name="aModelParameter">  a model parameter. </param>
        /// <param name="StoreLocal">       true to store local. </param>
        /// <remarks> Storing local does not access model during user input, only on initialization or on a reload cal or event.  Code must fetch the data at set the model parameters.  
        /// 		           Storing in model access the model and sets the model parameter each time a value is changed.</remarks>
        /// <exception cref="WaterSim_Exception"> if aModelParameter is not modelParamtype.mptInputBase</exception>
        public WaterSim_Base_InputTextBox(ModelParameterClass aModelParameter, bool StoreLocal)
            : base(aModelParameter, StoreLocal)
        {
            
        }
        //------------------------
        protected override bool _setmp(ModelParameterClass mp)
        {
            if (mp!=null)
            {
                if (mp.ParamType != modelParamtype.mptInputBase) throw new  Exception("Not InputBase");
                else
                {
                    Fmp = mp;
                    _isMPSet = true;
                    GetModelValue();
                     Fmp.ReloadEvent += ReloadEventHandler;
                }
            }
            return _isMPSet;
        }
        //------------------------
        protected override void SetValue(int value)
        {
            FValue = value;
            SetModelValue();
        }
        //------------------------
        protected override void GetModelValue()
        {
            if (Fmp!=null)
            {
                FValue = Fmp.Value;
                this.Text = FValue.ToString();
            }
        }
        //------------------------
        protected override void SetModelValue()
        {
            if ( (Fmp!=null)&(!FStoreLocal) )
            {
                Fmp.Value = FValue;
            }
        }
        //------------------------
        protected override bool ValueInRange(int value)
        {
            bool test = false;
            if (Fmp != null)
            {
                string errMessage = "";
                test = Fmp.isBaseValueInRange(value, ref errMessage);
                if (!test) MessageBox.Show(errMessage);
            }
            return test;
        }
        //-------------------------------------------------------
        //-------------------------------------------------------
    }
    //=================================================
    class WaterSim_Provider_InputTextBox : WaterSim_InputTextBox
    {
        protected ProviderIntArray FValues = new ProviderIntArray(0);
        protected eProvider Fprovider = (eProvider)0;
        protected bool FFillAll = false;
        //----------------------------------------------------------------------
  
        //----------------------------------------------------------------------
        public WaterSim_Provider_InputTextBox(ModelParameterClass aModelParameter, bool StoreLocal, eProvider p)
            : base(aModelParameter, StoreLocal)
        {
            Fprovider = p;
            GetModelValues();
        }
        //------------------------
        protected override bool _setmp(ModelParameterClass mp)
        {
            if (mp != null)
            {
                if (mp.ParamType != modelParamtype.mptInputProvider) throw new Exception("WS_Strings.wsModelParamMUstBeInputProvider");
                else
                {
                    Fmp = mp;
                    _isMPSet = true;
                    GetModelValue();
                    Fmp.ReloadEvent += ReloadEventHandler;
                }
            }
            return _isMPSet;
        }
        //------------------------
        protected override void SetValue(int value)
        {
            FValue = value;
            FValues[ProviderClass.index(Fprovider)] = value;
            SetModelValue();
        }
        //------------------------
        protected override void GetModelValue()
        {
            if (Fmp != null)
            {
                FValues = Fmp.ProviderProperty.getvalues();
                FValue = FValues[ProviderClass.index(Fprovider)];
                this.Text = FValue.ToString();
            }
        }
        //------------------------
        protected void GetModelValues()
        {
            if (Fmp != null)
            {
                FValues = Fmp.ProviderProperty.getvalues();
                FValue = FValues[ProviderClass.index(Fprovider)];
                this.Text = FValue.ToString();
            }
        }
        //------------------------
        protected override void SetModelValue()
        {
            if ( (Fmp != null)&(!FStoreLocal))   Fmp.ProviderProperty[ Fprovider] = FValue;
        }
        //------------------------
        protected void SetModelValues()
        {
            if ((Fmp != null)&(!FStoreLocal)) Fmp.ProviderProperty.setvalues(FValues);
        }
        //--------------------------------------------------------------------
        public void checkBoxCheckedChanged(object sender, EventArgs e)
        {
            // TO DO Add avlue checlk
            if (sender is CheckBox)
            {
                FFillAll = (sender as CheckBox).Checked;
                if (FFillAll)
                { 
                    for(int i=0;i<FValues.Values.Length;i++) FValues.Values[i] = FValue;
                 }
                else
                { GetModelValue(); }
            }
        }
        //--------------------------------------------------------------------
        public void SetAllButtonClicked(object sender, EventArgs e)
        {
            // TO DO Add avlue checlk
            if (Fmp != null)
            {
                string errMessage = "";
                string temp = "";
                bool test = false;
                bool founderror = false;
               
                for (int i = 0; i < FValues.Length; i++)
                {
                    temp = "";
                    test = Fmp.isProviderValueInRange(FValue, (eProvider)i, ref temp);
                    if (test)
                    {
                        FValues[i] = FValue;
                    }
                    else
                    {
                        founderror = true;
                        errMessage += "[" + temp +"]";
                    }
                }
                if (founderror)
                {
                    MessageBox.Show("Warning range errors, value=" +FValue.ToString()+" not set for "+  errMessage);
                }
                else
                    SetModelValues();
            }
        }
        //-------------------------------------------------------------
        public eProvider ModelParameterProvider
        {
            get { return Fprovider; }
            set
            {
                Fprovider = value;
                FValue = FValues[ProviderClass.index(Fprovider)];
                this.Text = FValue.ToString();
            }
        }
        //------------------------
        protected override bool ValueInRange(int value)
        {
            bool test = true;
            string errMessage = "";
            if (Fmp!=null) test = Fmp.isProviderValueInRange(value, Fprovider, ref errMessage);
            if (!test) MessageBox.Show(errMessage);
            return test;
        }
        //------------------------
        public ProviderIntArray GetValues()
        { return FValues; }
        //------------------------
        public void SetValues(ProviderIntArray values)
        { FValues = values; }
    }
    //==================================================================
    /**************************************************************************************************
     *  WaterSim Input Panel
     *  Creates a Panel that connects to a WaterSim instance, can be used to input data before 
     *  a Simulation is run.  Given WaterSim WS, and WaterSim_InputPanel WSP
     *      WS.initializeSimulation();
     *      WSP.SetAllModelValues();
     *      WS.SimulationAllYears();
     *      WS.stopSimulation()
     * 
     * 
     * 
     * *************************************************************************************************/
    public class WaterSim_InputPanel : Panel
    {
        const int inputTextBoxWidth = 50;
        const int inputTextBoxHeight = 20;

        public TableLayoutPanel InputtableLayoutPanel;
        public TableLayoutPanel ProvidertableLayoutPanel;
        ComboBox ProviderComboBox;
        List<WaterSim_Base_InputTextBox> BaseInputList = new List<WaterSim_Base_InputTextBox>();
        List<WaterSim_Provider_InputTextBox> ProviderInputList = new List<WaterSim_Provider_InputTextBox>();
        WaterSim WSim;

        //---------------------------------
        public WaterSim_InputPanel() : base() { }
        //---------------------------------

        public WaterSim_InputPanel(WaterSim WS,int width, int height, int top, int left, Control Owner)
            : base()
        {
            WSim = WS;
            LoadControls(width, height, top, left,Owner);
        }
        //---------------------------------
        public WaterSim WaterSimModel 
        {
            get  { return WSim; }
            set
            {
                if(value!=null)
                {
                    WSim = value;
                    
                    LoadControls(this.Width, this.Height, this.Top, this.Left, this.Parent);
                }
             }
        } 
        internal void LoadControls(int width, int height, int top, int left, Control Owner )
        {
            Label InputLabel;
            ModelParameterClass mp;
            WaterSim_Base_InputTextBox biTextBox;
            WaterSim_Provider_InputTextBox piTextBox;
            
            Button setallbutton;

            int Row = 0;
            int ipheight, ipwidth, tlpheight, tlpwidth;
            int ipTop = top;
            int ipLeft = left;
            int tlpTop = 40;
            int HeaderLabelTop, HeaderSize;



            ipheight = height;
            ipwidth = width;
            tlpheight = ipheight - tlpTop - 15;
            tlpwidth = (ipwidth / 2) - 25;

            // Create Input Panel
            //Panel ModelInputPanel = new Panel();


            this.Top = ipTop;
            this.Left = ipLeft;
            this.Size = new Size(ipwidth, ipheight);
            this.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.Anchor = (AnchorStyles.Bottom | AnchorStyles.Top);

           // Owner.Controls.Add(this);

            HeaderSize = Convert.ToInt16(Owner.Font.SizeInPoints * 5) + 2;
            HeaderLabelTop = (HeaderSize / 2)+2;
            tlpTop = HeaderSize;


            // Add providerbox
            InputLabel = new Label();
            InputLabel.AutoSize = true;
            InputLabel.Top = 4;
            InputLabel.Left = (ipwidth / 2) + 3;
            InputLabel.Text = "Provider ";
            InputLabel.Font = new Font(this.Font, System.Drawing.FontStyle.Bold);
            this.Controls.Add(InputLabel);

            ProviderComboBox = new ComboBox();
            ProviderComboBox.Top = 2;
            ProviderComboBox.Left = InputLabel.Right + 2;
            ProviderComboBox.Width = (ipwidth / 2) - (InputLabel.Width + 8);
            foreach (eProvider p in ProviderClass.providers())
                ProviderComboBox.Items.Add(ProviderClass.Label(p));
            ProviderComboBox.SelectedIndex = 0;
            ProviderComboBox.SelectedIndexChanged += ProviderComboBox_SelectedIndexChanged;

            this.Controls.Add(ProviderComboBox);

            // Create Base Input Table
            InputtableLayoutPanel = new TableLayoutPanel();
            InputtableLayoutPanel.Size = new Size(tlpwidth, tlpheight);
            InputtableLayoutPanel.Top = tlpTop;
            InputtableLayoutPanel.Left = 1;
            InputtableLayoutPanel.ColumnCount = 2;
            InputtableLayoutPanel.RowCount = WSim.ParamManager.NumberOfParameters(modelParamtype.mptInputBase);
            InputtableLayoutPanel.AutoScroll = true;
            InputtableLayoutPanel.Anchor = (AnchorStyles.Bottom | AnchorStyles.Top);
            ColumnStyle style = new ColumnStyle();
            style.SizeType = SizeType.Percent;
            style.Width = 0.80F;
            InputtableLayoutPanel.ColumnStyles.Add(style);
            style = new ColumnStyle();
            style.SizeType = SizeType.Percent;
            style.Width = 0.20F;
            InputtableLayoutPanel.ColumnStyles.Add(style);
            this.Controls.Add(InputtableLayoutPanel);

          Row = 0;
       // Add the Base Inputs to it
            foreach (eModelParam p in WSim.ParamManager.ModelParameters())
            {
                //InputtableLayoutPanel
                mp = WSim.ParamManager.Model_Parameter(p);
                if (mp.ParamType == modelParamtype.mptInputBase)
                {
                    InputLabel = new Label();
                    InputLabel.AutoSize = true;
                    InputLabel.Text = mp.Label;
                    InputtableLayoutPanel.Controls.Add(InputLabel);
                    InputtableLayoutPanel.SetCellPosition(InputLabel, new TableLayoutPanelCellPosition(0, Row));

                    biTextBox = new WaterSim_Base_InputTextBox(mp, false);
                    biTextBox.Size = new Size(inputTextBoxWidth,inputTextBoxHeight);
                    InputtableLayoutPanel.Controls.Add(biTextBox);
                    InputtableLayoutPanel.SetColumn(biTextBox, 1);
                    InputtableLayoutPanel.SetRow(biTextBox, Row);
                    BaseInputList.Add(biTextBox);

                    Row++;
                }
            }
            //OK now put a label down for columns
           
            float ColWidth = InputtableLayoutPanel.ColumnStyles[0].Width * InputtableLayoutPanel.Width;

            InputLabel = new Label();
            InputLabel.AutoSize = true;
            InputLabel.Top = HeaderLabelTop;
            InputLabel.Left = 0;
            InputLabel.Text = "Base Simulation Parameters";
            InputLabel.Font = new Font(Font, System.Drawing.FontStyle.Bold);
            this.Controls.Add(InputLabel);

            InputLabel = new Label();
            InputLabel.AutoSize = true;
            InputLabel.Top = HeaderLabelTop;
            InputLabel.Left = Convert.ToInt16(ColWidth);
            InputLabel.Text = "Value";
            InputLabel.Font = new Font(Font, System.Drawing.FontStyle.Bold);
            this.Controls.Add(InputLabel);

            //-------------------------------------------
            // Create the Provider Table
            ProvidertableLayoutPanel = new TableLayoutPanel();
            ProvidertableLayoutPanel.Size = new Size(tlpwidth+20, tlpheight);
            ProvidertableLayoutPanel.Top = tlpTop;
            ProvidertableLayoutPanel.Left = InputtableLayoutPanel.Left+  InputtableLayoutPanel.Width+4 ;
            ProvidertableLayoutPanel.ColumnCount = 3;
            ProvidertableLayoutPanel.AutoScroll = true;
            ProvidertableLayoutPanel.Anchor = (AnchorStyles.Bottom | AnchorStyles.Top);
            ProvidertableLayoutPanel.RowCount = WSim.ParamManager.NumberOfParameters(modelParamtype.mptInputBase);

            style = new ColumnStyle();
            style.SizeType = SizeType.Percent;
            style.Width = 0.70F;
            ProvidertableLayoutPanel.ColumnStyles.Add(style);
            style = new ColumnStyle();
            style.SizeType = SizeType.Percent;
            style.Width = 0.20F;
            ProvidertableLayoutPanel.ColumnStyles.Add(style);
            style = new ColumnStyle();
            style.SizeType = SizeType.Percent;
            style.Width = 0.10F;
            ProvidertableLayoutPanel.ColumnStyles.Add(style);

            this.Controls.Add(ProvidertableLayoutPanel);

            // Add the Provider Inouts to it
            Row = 1;
            foreach (eModelParam p in WSim.ParamManager.ModelParameters())
            {
                InputLabel = new Label();
                InputLabel.AutoSize = true;

                
                //InputtableLayoutPanel
                mp = WSim.ParamManager.Model_Parameter(p);
                if (mp.ParamType == modelParamtype.mptInputProvider)
                {
                    InputLabel = new Label();
                    InputLabel.AutoSize = true;
                    InputLabel.Text = mp.Label;
                    ProvidertableLayoutPanel.Controls.Add(InputLabel);
                    ProvidertableLayoutPanel.SetColumn(InputLabel, 0);
                    ProvidertableLayoutPanel.SetRow(InputLabel, Row);

                    piTextBox = new WaterSim_Provider_InputTextBox(mp, false, ProviderClass.provider(0));
                    piTextBox.Size = new Size(inputTextBoxWidth, inputTextBoxHeight);
                    ProvidertableLayoutPanel.Controls.Add(piTextBox);
                    ProvidertableLayoutPanel.SetColumn(piTextBox, 1);
                    ProvidertableLayoutPanel.SetRow(piTextBox, Row);
                    ProviderInputList.Add(piTextBox);

                    //SetAllButtonClicked
                    setallbutton  = new Button();
                    setallbutton.Font = new Font(Font.FontFamily, 8.0F, System.Drawing.FontStyle.Bold);
                    setallbutton.Text = "Set All";
                    setallbutton.AutoSize = true;
                    setallbutton.Click += piTextBox.SetAllButtonClicked;
                    ProvidertableLayoutPanel.Controls.Add(setallbutton);
                    ProvidertableLayoutPanel.SetColumn(setallbutton, 2);
                    ProvidertableLayoutPanel.SetRow(setallbutton, Row);

                    //chckbox = new CheckBox();
                    //chckbox.Text = "";
                    //chckbox.CheckedChanged += piTextBox.checkBoxCheckedChanged;
                    //ProvidertableLayoutPanel.Controls.Add(chckbox);
                    //ProvidertableLayoutPanel.SetColumn(chckbox, 2);
                    //ProvidertableLayoutPanel.SetRow(chckbox, Row);

                    Row++;
                }

 
                InputLabel = new Label();
                InputLabel.AutoSize = true;
                InputLabel.Top = HeaderLabelTop;
                InputLabel.Left = ProvidertableLayoutPanel.Left;
                InputLabel.Text = "Provider Simulation Parameters";
                InputLabel.Font = new Font(Font, System.Drawing.FontStyle.Bold);
                this.Controls.Add(InputLabel);

                ColWidth = ProvidertableLayoutPanel.ColumnStyles[0].Width * ProvidertableLayoutPanel.Width;
                InputLabel = new Label();
                InputLabel.AutoSize = true;
                InputLabel.Top = HeaderLabelTop;
                InputLabel.Left = Convert.ToInt16(ColWidth) + ProvidertableLayoutPanel.Left-8;
                InputLabel.Text = "Value";
                InputLabel.Font = new Font(Font, System.Drawing.FontStyle.Bold);
                this.Controls.Add(InputLabel);

//                ColWidth += ProvidertableLayoutPanel.ColumnStyles[1].Width * ProvidertableLayoutPanel.Width;
                InputLabel = new Label();
                InputLabel.AutoSize = true;
                InputLabel.Top = HeaderLabelTop;
                InputLabel.Left = Convert.ToInt16(ColWidth) + ProvidertableLayoutPanel.Left + inputTextBoxWidth;
                InputLabel.Text = "Set  All";
                InputLabel.Font = new Font(Font, System.Drawing.FontStyle.Bold);
                this.Controls.Add(InputLabel);

                //InputLabel = new Label();
                //InputLabel.AutoSize = true;
                //InputLabel.Top = HeaderLabelTop;
                //InputLabel.Left = (ipwidth / 2) + 3;
                //InputLabel.Text = "Provider Simulation Parameters                      Value    SetAll";
                //InputLabel.Font = new Font(Font, System.Drawing.FontStyle.Bold);
                //this.Controls.Add(InputLabel);
            }
        }

        //-------------------------------------------------------------------------------------------------------
        private void ProviderComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            eProvider pSelected = ProviderClass.provider(ProviderComboBox.SelectedIndex);
            foreach (WaterSim_Provider_InputTextBox piTextBox in ProviderInputList)
                piTextBox.ModelParameterProvider = pSelected;

        }
        //-------------------------------------------------------------------------------------------------------
        public SimulationInputs GetAllValues()
        {
            SimulationInputs si = WSim.ParamManager.NewSimulationInputs();

           int cnt = InputtableLayoutPanel.Controls.Count;
           int index = 0;
           for (int i = 0; i < cnt; i++)
           {
             if (InputtableLayoutPanel.Controls[i] is WaterSim_Base_InputTextBox)
             {

                 WaterSim_Base_InputTextBox bic;
                 bic = InputtableLayoutPanel.Controls[i] as WaterSim_Base_InputTextBox;
                 si.BaseInput[index] = bic.Value;
                 index++;
              }
           }
           cnt =ProvidertableLayoutPanel.Controls.Count;
          
           index = 0;
           for (int i = 0; i < cnt; i++)
           {
               if (ProvidertableLayoutPanel.Controls[i] is WaterSim_Provider_InputTextBox)
               {
                   WaterSim_Provider_InputTextBox pic;
                   pic = ProvidertableLayoutPanel.Controls[i] as WaterSim_Provider_InputTextBox;
                   //pdata = si.ProviderInput[index];
                   si.ProviderInput.Values[index] = pic.GetValues();
                   //pdata.Values = pic.GetValues();
                   index++;
               }
           }
           return si;
        }
        //-------------------------------------------------------------------------------------------------------
        public void  SetAllValues(SimulationInputs si)
        {
            int cnt = InputtableLayoutPanel.Controls.Count;
            int index = 0;
            for (int i = 0; i < cnt; i++)
            {
                if (InputtableLayoutPanel.Controls[i] is WaterSim_Base_InputTextBox)
                {
                    WaterSim_Base_InputTextBox bic;
                    bic = InputtableLayoutPanel.Controls[i] as WaterSim_Base_InputTextBox;
                    bic.Value = si.BaseInput[index];
                    index++;
                }
            }
            cnt = ProvidertableLayoutPanel.Controls.Count;
            index = 0;
            ProviderIntArray pdata;
            for (int i = 0; i < cnt; i++)
            {
                if (ProvidertableLayoutPanel.Controls[i] is WaterSim_Provider_InputTextBox)
                {
                    WaterSim_Provider_InputTextBox pic;
                    pic = ProvidertableLayoutPanel.Controls[i] as WaterSim_Provider_InputTextBox;
                    pdata = si.ProviderInput[index];
                    pic.SetValues(pdata);
                }
            }
        }
        //-------------------------------------------------------------------------------------------------------
        public void Refresh_Inputs()
        {
            int cnt = InputtableLayoutPanel.Controls.Count;
            for (int i = 0; i < cnt; i++)
            {
                if (InputtableLayoutPanel.Controls[i] is WaterSim_Base_InputTextBox)
                {
                    WaterSim_Base_InputTextBox bic;
                    bic = InputtableLayoutPanel.Controls[i] as WaterSim_Base_InputTextBox;
                    bic.ReloadModelValue();
                }
            }
            cnt = ProvidertableLayoutPanel.Controls.Count;
            for (int i = 0; i < cnt; i++)
            {
                if (ProvidertableLayoutPanel.Controls[i] is WaterSim_Provider_InputTextBox)
                {
                    WaterSim_Provider_InputTextBox pic;
                    pic = ProvidertableLayoutPanel.Controls[i] as WaterSim_Provider_InputTextBox;
                    pic.ReloadModelValue();
                }
            }
        }
        //-------------------------------------------------------------------------------------------------------
    }
    public class ModelParmeterClassConverter : System.ComponentModel.TypeConverter
    {
        public override bool CanConvertFrom(System.ComponentModel.ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string))
                return true;
            else
               return base.CanConvertFrom(context, sourceType);
        }
        public override bool CanConvertTo(System.ComponentModel.ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(string))
            {
                return true;
            }
            else
               return base.CanConvertTo(context, destinationType);
        }
        public override object ConvertTo(System.ComponentModel.ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if ((destinationType == typeof(string)) & (value is ModelParameterClass))
            {
                return value.ToString();
            }
            else
                return base.ConvertTo(context, culture, value, destinationType);
        }
        public override object ConvertFrom(System.ComponentModel.ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {   
            return base.ConvertFrom(context, culture, value);
        }

    }


}
